@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Plus42 -- an enhanced HP-42S calculator simulator
- * Copyright (C) 2004-2022  Thomas Okken
+ * Copyright (C) 2004-2025  Thomas Okken
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -153,7 +153,7 @@ int docmd_path(arg_struct *arg) {
             dir = parent;
         }
         path = std::string("HOME", 4) + path;
-        vartype *v = new_string(path.c_str(), path.length());
+        vartype *v = new_string(path.c_str(), (int) path.length());
         if (v == NULL)
             return ERR_INSUFFICIENT_MEMORY;
         return recall_result(v);
@@ -541,7 +541,7 @@ static int ref_move_copy(bool copy) {
                     if (new_prgms[j].capacity == cwd->id && new_prgms[j].size == i)
                         goto skip3;
             }
-            if (first && cwd->prgms[i].text[0] == CMD_END) {
+            if (first && cwd->prgms[i].is_end(0)) {
                 // There is an empty program at the end of the current
                 // directory, while we are moving or copying programs in:
                 // remove the empty program, similarly to how we treat
@@ -552,6 +552,7 @@ static int ref_move_copy(bool copy) {
             new_prgms[new_prgms_count].capacity = cwd->id;
             new_prgms[new_prgms_count].size = i;
             new_prgms[new_prgms_count].text = NULL;
+            count_embed_references(cwd, new_prgms_count, true);
             new_prgms_count++;
             skip3:
             first = false;
@@ -767,7 +768,7 @@ int docmd_prall(arg_struct *arg) {
         return ERR_NONE;
     if (!flags.f.printer_exists)
         return ERR_PRINTING_IS_DISABLED;
-    shell_annunciators(-1, -1, 1, -1, -1, -1);
+    set_annunciators(-1, -1, 1, -1, -1, -1);
     print_text(NULL, 0, true);
     print_text("HOME: Dir", 9, true);
     prall_dir = root;
@@ -779,7 +780,7 @@ int docmd_prall(arg_struct *arg) {
 
 static int prall_worker(bool interrupted) {
     if (interrupted) {
-        shell_annunciators(-1, -1, 0, -1, -1, -1);
+        set_annunciators(-1, -1, 0, -1, -1, -1);
         return ERR_STOP;
     }
     char buf[100];
@@ -876,7 +877,7 @@ static int prall_worker(bool interrupted) {
     } else {
         if (prall_dir->parent == NULL) {
             done:
-            shell_annunciators(-1, -1, 0, -1, -1, -1);
+            set_annunciators(-1, -1, 0, -1, -1, -1);
             return ERR_NONE;
         } else {
             for (int i = 0; i < prall_dir->parent->children_count; i++)
@@ -996,6 +997,41 @@ int docmd_setds(arg_struct *arg) {
     return ERR_NONE;
 }
 
+int docmd_1line(arg_struct *arg) {
+    mode_multi_line = false;
+    return ERR_NONE;
+}
+
+int docmd_nline(arg_struct *arg) {
+    mode_multi_line = true;
+    return ERR_NONE;
+}
+
+int docmd_ltop(arg_struct *arg) {
+    mode_lastx_top = !mode_lastx_top;
+    return ERR_NONE;
+}
+
+int docmd_atop(arg_struct *arg) {
+    mode_alpha_top = !mode_alpha_top;
+    return ERR_NONE;
+}
+
+int docmd_hflags(arg_struct *arg) {
+    mode_header_flags = !mode_header_flags;
+    return ERR_NONE;
+}
+
+int docmd_hpolar(arg_struct *arg) {
+    mode_header_polar = !mode_header_polar;
+    return ERR_NONE;
+}
+
+int docmd_stk(arg_struct *arg) {
+    mode_matedit_stk = !mode_matedit_stk;
+    return ERR_NONE;
+}
+
 int docmd_dirs(arg_struct *arg) {
     set_menu(MENULEVEL_AUX, MENU_CATALOG);
     set_cat_section_no_top(CATSECT_DIRS);
@@ -1004,7 +1040,7 @@ int docmd_dirs(arg_struct *arg) {
 }
 
 int docmd_dir_fcn(arg_struct *arg) {
-    set_plainmenu(MENU_DIR_FCN1);
+    set_plainmenu(MENU_DIR_FCN1, NULL, 0);
     return ERR_NONE;
 }
 
@@ -1016,7 +1052,7 @@ int docmd_units(arg_struct *arg) {
 }
 
 int docmd_unit_fcn(arg_struct *arg) {
-    set_plainmenu(MENU_UNIT_FCN);
+    set_plainmenu(MENU_UNIT_FCN1, NULL, 0);
     return ERR_NONE;
 }
 
@@ -1177,13 +1213,12 @@ static int do_i_pct_yr(phloat n, phloat pv, phloat pmt, phloat fv, phloat p_yr,
         phloat f = 0;
         while (true) {
             phloat eps, f0 = f;
-            if (1 + i * i == 1) {
-                phloat a = (pv + fv) / n;
-                phloat b = pv - fv;
-                phloat fp = (a + b) / 2;
-                phloat fpp = (n * n - 1) * a / 6;
-                f = a + pmt + fp * i;
-                fp += fpp * i;
+            if (1 + n * i * i == 1) {
+                f = (pv + fv + n * pmt) / n;        // f(0)
+                phloat a = f - pmt;                 // f(0) - pmt
+                phloat b = (n * n - 1) * a / 6 * i; // f''(0)*i    
+                phloat fp = (pv - fv + a) / 2 + b;  // f'(0) + f''(0)*i
+                f = f + (fp - b / 2) * i;           // f(0) + f'(0)*i + f''(0)/2*i^2
                 i += (eps = -f / fp);
             } else {
                 phloat x = i / expm1(n * log1p(i));
@@ -1205,7 +1240,7 @@ static int do_i_pct_yr(phloat n, phloat pv, phloat pmt, phloat fv, phloat p_yr,
             if (f == 0 || (f > 0) != (f0 > 0))
                 break;
             if (fabs(f) >= fabs(f0)) {
-                if (i == i - eps / 1000) {
+                if (i + eps * 1e-6 == i) {
                     i -= eps / 2;
                     break;
                 } else
@@ -1380,7 +1415,7 @@ static void show_tvm_message() {
 
 int docmd_tvm(arg_struct *arg) {
     if (flags.f.prgm_mode) {
-        set_plainmenu(MENU_TVM_PRGM1);
+        set_plainmenu(MENU_TVM_PRGM1, NULL, 0);
         return ERR_NONE;
     }
 
@@ -1417,7 +1452,7 @@ int docmd_eqn(arg_struct *arg) {
 }
 
 int docmd_eqn_fcn(arg_struct *arg) {
-    set_plainmenu(MENU_EQN_FCN);
+    set_plainmenu(MENU_EQN_FCN1, NULL, 0);
     return ERR_NONE;
 }
 
@@ -2063,7 +2098,7 @@ static int tgo_worker(bool interrupted) {
     int err = ERR_STOP;
     if (interrupted) {
         done:
-        shell_annunciators(-1, -1, 0, -1, -1, -1);
+        set_annunciators(-1, -1, 0, -1, -1, -1);
         free_vartype((vartype *) tgo_rm);
         return err;
     }
@@ -2110,7 +2145,7 @@ int docmd_tgo(arg_struct *arg) {
     if (!flags.f.printer_exists)
         return ERR_PRINTING_IS_DISABLED;
 
-    shell_annunciators(-1, -1, 1, -1, -1, -1);
+    set_annunciators(-1, -1, 1, -1, -1, -1);
 
     print_text(NULL, 0, true);
     char buf[50];
@@ -2130,7 +2165,7 @@ int docmd_tgo(arg_struct *arg) {
         nomem:
         err = ERR_INSUFFICIENT_MEMORY;
         done:
-        shell_annunciators(-1, -1, 0, -1, -1, -1);
+        set_annunciators(-1, -1, 0, -1, -1, -1);
         free_vartype((vartype *) rm);
         return err;
     }
