@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Plus42 -- an enhanced HP-42S calculator simulator
- * Copyright (C) 2004-2022  Thomas Okken
+ * Copyright (C) 2004-2025  Thomas Okken
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2,
@@ -72,203 +72,6 @@ int assert_numeric(const vartype *v) {
         return ERR_INVALID_TYPE;
 }
 
-int generic_div(const vartype *px, const vartype *py, int (*completion)(int, vartype *)) {
-    if (px->type == TYPE_UNIT && (py->type == TYPE_UNIT || py->type == TYPE_REAL)
-            || px->type == TYPE_REAL && py->type == TYPE_UNIT) {
-        vartype *dst;
-        int error = unit_div(px, py, &dst);
-        return completion(error, dst);
-    } else if ((px->type == TYPE_REALMATRIX || px->type == TYPE_COMPLEXMATRIX)
-            && (py->type == TYPE_REALMATRIX || py->type == TYPE_COMPLEXMATRIX)){
-        return linalg_div(py, px, completion);
-    } else {
-        vartype *dst;
-        int error = map_binary(px, py, &dst, div_rr, div_rc, div_cr, div_cc);
-        return completion(error, dst);
-    }
-}
-
-int generic_mul(const vartype *px, const vartype *py, int (*completion)(int, vartype *)) {
-    if (px->type == TYPE_UNIT && (py->type == TYPE_UNIT || py->type == TYPE_REAL)
-            || px->type == TYPE_REAL && py->type == TYPE_UNIT) {
-        vartype *dst;
-        int error = unit_mul(px, py, &dst);
-        return completion(error, dst);
-    } else if ((px->type == TYPE_REALMATRIX || px->type == TYPE_COMPLEXMATRIX)
-            && (py->type == TYPE_REALMATRIX || py->type == TYPE_COMPLEXMATRIX)){
-        return linalg_mul(py, px, completion);
-    } else {
-        vartype *dst;
-        int error = map_binary(px, py, &dst, mul_rr, mul_rc, mul_cr, mul_cc);
-        return completion(error, dst);
-    }
-}
-
-int generic_sub(const vartype *px, const vartype *py, vartype **dst) {
-    if (px->type == TYPE_REAL && py->type == TYPE_REAL) {
-        vartype_real *x = (vartype_real *) px;
-        vartype_real *y = (vartype_real *) py;
-        phloat res = y->x - x->x;
-        int inf = p_isinf(res);
-        if (inf != 0) {
-            if (flags.f.range_error_ignore)
-                res = inf == 1 ? POS_HUGE_PHLOAT : NEG_HUGE_PHLOAT;
-            else
-                return ERR_OUT_OF_RANGE;
-        }
-        *dst = new_real(res);
-        if (*dst == NULL)
-            return ERR_INSUFFICIENT_MEMORY;
-        else
-            return ERR_NONE;
-    } else if ((px->type == TYPE_REAL || px->type == TYPE_UNIT)
-            && (py->type == TYPE_REAL || py->type == TYPE_UNIT)) {
-        return unit_sub(px, py, dst);
-    } else if (px->type == TYPE_COMPLEX && py->type == TYPE_COMPLEX) {
-        vartype_complex *x = (vartype_complex *) px;
-        vartype_complex *y = (vartype_complex *) py;
-        int inf;
-        phloat re, im;
-        re = y->re - x->re;
-        inf = p_isinf(re);
-        if (inf != 0) {
-            if (flags.f.range_error_ignore)
-                re = inf == 1 ? POS_HUGE_PHLOAT : NEG_HUGE_PHLOAT;
-            else
-                return ERR_OUT_OF_RANGE;
-        }
-        im = y->im - x->im;
-        inf = p_isinf(im);
-        if (inf != 0) {
-            if (flags.f.range_error_ignore)
-                im = inf == 1 ? POS_HUGE_PHLOAT : NEG_HUGE_PHLOAT;
-            else
-                return ERR_OUT_OF_RANGE;
-        }
-        *dst = new_complex(re, im);
-        if (*dst == NULL)
-            return ERR_INSUFFICIENT_MEMORY;
-        else
-            return ERR_NONE;
-    } else if (px->type == TYPE_REAL && py->type == TYPE_COMPLEX) {
-        vartype_real *x = (vartype_real *) px;
-        vartype_complex *y = (vartype_complex *) py;
-        int inf;
-        phloat re = y->re - x->x;
-        inf = p_isinf(re);
-        if (inf != 0) {
-            if (flags.f.range_error_ignore)
-                re = inf == 1 ? POS_HUGE_PHLOAT : NEG_HUGE_PHLOAT;
-            else
-                return ERR_OUT_OF_RANGE;
-        }
-        *dst = new_complex(re, y->im);
-        if (*dst == NULL)
-            return ERR_INSUFFICIENT_MEMORY;
-        else
-            return ERR_NONE;
-    } else if (px->type == TYPE_COMPLEX && py->type == TYPE_REAL) {
-        vartype_complex *x = (vartype_complex *) px;
-        vartype_real *y = (vartype_real *) py;
-        phloat re = y->x - x->re;
-        int inf = p_isinf(re);
-        if (inf != 0) {
-            if (flags.f.range_error_ignore)
-                re = inf == 1 ? POS_HUGE_PHLOAT : NEG_HUGE_PHLOAT;
-            else
-                return ERR_OUT_OF_RANGE;
-        }
-        *dst = new_complex(re, -x->im);
-        if (*dst == NULL)
-            return ERR_INSUFFICIENT_MEMORY;
-        else
-            return ERR_NONE;
-    } else
-        return map_binary(px, py, dst, sub_rr, sub_rc, sub_cr, sub_cc);
-}
-
-int generic_add(const vartype *px, const vartype *py, vartype **dst) {
-    if (px->type == TYPE_REAL && py->type == TYPE_REAL) {
-        vartype_real *x = (vartype_real *) px;
-        vartype_real *y = (vartype_real *) py;
-        phloat res = y->x + x->x;
-        int inf = p_isinf(res);
-        if (inf != 0) {
-            if (flags.f.range_error_ignore)
-                res = inf == 1 ? POS_HUGE_PHLOAT : NEG_HUGE_PHLOAT;
-            else
-                return ERR_OUT_OF_RANGE;
-        }
-        *dst = new_real(res);
-        if (*dst == NULL)
-            return ERR_INSUFFICIENT_MEMORY;
-        else
-            return ERR_NONE;
-    } else if ((px->type == TYPE_REAL || px->type == TYPE_UNIT)
-            && (py->type == TYPE_REAL || py->type == TYPE_UNIT)) {
-        return unit_add(px, py, dst);
-    } else if (px->type == TYPE_COMPLEX && py->type == TYPE_COMPLEX) {
-        vartype_complex *x = (vartype_complex *) px;
-        vartype_complex *y = (vartype_complex *) py;
-        int inf;
-        phloat re, im;
-        re = x->re + y->re;
-        inf = p_isinf(re);
-        if (inf != 0) {
-            if (flags.f.range_error_ignore)
-                re = inf == 1 ? POS_HUGE_PHLOAT : NEG_HUGE_PHLOAT;
-            else
-                return ERR_OUT_OF_RANGE;
-        }
-        im = x->im + y->im;
-        inf = p_isinf(im);
-        if (inf != 0) {
-            if (flags.f.range_error_ignore)
-                im = inf == 1 ? POS_HUGE_PHLOAT : NEG_HUGE_PHLOAT;
-            else
-                return ERR_OUT_OF_RANGE;
-        }
-        *dst = new_complex(re, im);
-        if (*dst == NULL)
-            return ERR_INSUFFICIENT_MEMORY;
-        else
-            return ERR_NONE;
-    } else if (px->type == TYPE_REAL && py->type == TYPE_COMPLEX) {
-        vartype_real *x = (vartype_real *) px;
-        vartype_complex *y = (vartype_complex *) py;
-        phloat re = y->re + x->x;
-        int inf = p_isinf(re);
-        if (inf != 0) {
-            if (flags.f.range_error_ignore)
-                re = inf == 1 ? POS_HUGE_PHLOAT : NEG_HUGE_PHLOAT;
-            else
-                return ERR_OUT_OF_RANGE;
-        }
-        *dst = new_complex(re, y->im);
-        if (*dst == NULL)
-            return ERR_INSUFFICIENT_MEMORY;
-        else
-            return ERR_NONE;
-    } else if (px->type == TYPE_COMPLEX && py->type == TYPE_REAL) {
-        vartype_complex *x = (vartype_complex *) px;
-        vartype_real *y = (vartype_real *) py;
-        phloat re = x->re + y->x;
-        int inf = p_isinf(re);
-        if (inf != 0) {
-            if (flags.f.range_error_ignore)
-                re = inf == 1 ? POS_HUGE_PHLOAT : NEG_HUGE_PHLOAT;
-            else
-                return ERR_OUT_OF_RANGE;
-        }
-        *dst = new_complex(re, x->im);
-        if (*dst == NULL)
-            return ERR_INSUFFICIENT_MEMORY;
-        else
-            return ERR_NONE;
-    } else
-        return map_binary(px, py, dst, add_rr, add_rc, add_cr, add_cc);
-}
-
 int generic_rcl(arg_struct *arg, vartype **dst, bool must_be_writable) {
     int err;
     if (arg->type == ARGTYPE_IND_NUM
@@ -638,7 +441,7 @@ int generic_sto(arg_struct *arg, char operation) {
                     free_vartype(newval);
                 return err;
             } else {
-                vloc loc = lookup_var(arg->val.text, arg->length, false, true);
+                vloc loc = lookup_var(arg->val.text, arg->length);
                 if (loc.not_found())
                     return ERR_NONEXISTENT;
                 if (!loc.writable())
@@ -1129,7 +932,7 @@ int map_binary(const vartype *src1, const vartype *src2, vartype **dst,
     }
 }
 
-int div_rr(phloat x, phloat y, phloat *z) {
+static int div_rr(phloat x, phloat y, phloat *z) {
     phloat r;
     int inf;
     if (x == 0)
@@ -1146,7 +949,7 @@ int div_rr(phloat x, phloat y, phloat *z) {
     return ERR_NONE;
 }
 
-int div_rc(phloat x, phloat yre, phloat yim, phloat *zre, phloat *zim) {
+static int div_rc(phloat x, phloat yre, phloat yim, phloat *zre, phloat *zim) {
     phloat rre, rim;
     int inf;
     if (x == 0)
@@ -1177,7 +980,7 @@ int div_rc(phloat x, phloat yre, phloat yim, phloat *zre, phloat *zim) {
  * http://forge.scilab.org/index.php/p/compdiv/source/tree/21/doc/improved_cdiv.pdf
  */
 
-int div_cr(phloat xre, phloat xim, phloat y, phloat *zre, phloat *zim) {
+static int div_cr(phloat xre, phloat xim, phloat y, phloat *zre, phloat *zim) {
     phloat r, t, rre, rim;
     int inf;
     if (xre == 0 && xim == 0)
@@ -1222,7 +1025,7 @@ int div_cr(phloat xre, phloat xim, phloat y, phloat *zre, phloat *zim) {
     return ERR_NONE;
 }
 
-int div_cc(phloat xre, phloat xim, phloat yre, phloat yim,
+static int div_cc(phloat xre, phloat xim, phloat yre, phloat yim,
                                                     phloat *zre, phloat *zim) {
     phloat r, t, rre, rim;
     int inf;
@@ -1268,7 +1071,7 @@ int div_cc(phloat xre, phloat xim, phloat yre, phloat yim,
     return ERR_NONE;
 }
 
-int mul_rr(phloat x, phloat y, phloat *z) {
+static int mul_rr(phloat x, phloat y, phloat *z) {
     phloat r;
     int inf;
     r = y * x;
@@ -1283,7 +1086,7 @@ int mul_rr(phloat x, phloat y, phloat *z) {
     return ERR_NONE;
 }
 
-int mul_rc(phloat x, phloat yre, phloat yim, phloat *zre, phloat *zim) {
+static int mul_rc(phloat x, phloat yre, phloat yim, phloat *zre, phloat *zim) {
     phloat rre, rim;
     int inf;
     rre = yre * x;
@@ -1307,7 +1110,7 @@ int mul_rc(phloat x, phloat yre, phloat yim, phloat *zre, phloat *zim) {
     return ERR_NONE;
 }
 
-int mul_cr(phloat xre, phloat xim, phloat y, phloat *zre, phloat *zim) {
+static int mul_cr(phloat xre, phloat xim, phloat y, phloat *zre, phloat *zim) {
     phloat rre, rim;
     int inf;
     rre = y * xre;
@@ -1331,7 +1134,7 @@ int mul_cr(phloat xre, phloat xim, phloat y, phloat *zre, phloat *zim) {
     return ERR_NONE;
 }
 
-int mul_cc(phloat xre, phloat xim, phloat yre, phloat yim,
+static int mul_cc(phloat xre, phloat xim, phloat yre, phloat yim,
                                                     phloat *zre, phloat *zim) {
     phloat rre, rim;
     int inf;
@@ -1356,7 +1159,7 @@ int mul_cc(phloat xre, phloat xim, phloat yre, phloat yim,
     return ERR_NONE;
 }
 
-int sub_rr(phloat x, phloat y, phloat *z) {
+static int sub_rr(phloat x, phloat y, phloat *z) {
     phloat r;
     int inf;
     r = y - x;
@@ -1371,7 +1174,7 @@ int sub_rr(phloat x, phloat y, phloat *z) {
     return ERR_NONE;
 }
 
-int sub_rc(phloat x, phloat yre, phloat yim, phloat *zre, phloat *zim) {
+static int sub_rc(phloat x, phloat yre, phloat yim, phloat *zre, phloat *zim) {
     phloat rre;
     int inf;
     rre = yre - x;
@@ -1387,7 +1190,7 @@ int sub_rc(phloat x, phloat yre, phloat yim, phloat *zre, phloat *zim) {
     return ERR_NONE;
 }
 
-int sub_cr(phloat xre, phloat xim, phloat y, phloat *zre, phloat *zim) {
+static int sub_cr(phloat xre, phloat xim, phloat y, phloat *zre, phloat *zim) {
     phloat rre;
     int inf;
     rre = y - xre;
@@ -1403,7 +1206,7 @@ int sub_cr(phloat xre, phloat xim, phloat y, phloat *zre, phloat *zim) {
     return ERR_NONE;
 }
 
-int sub_cc(phloat xre, phloat xim, phloat yre, phloat yim,
+static int sub_cc(phloat xre, phloat xim, phloat yre, phloat yim,
                                                     phloat *zre, phloat *zim) {
     phloat rre, rim;
     int inf;
@@ -1428,7 +1231,7 @@ int sub_cc(phloat xre, phloat xim, phloat yre, phloat yim,
     return ERR_NONE;
 }
 
-int add_rr(phloat x, phloat y, phloat *z) {
+static int add_rr(phloat x, phloat y, phloat *z) {
     phloat r;
     int inf;
     r = y + x;
@@ -1443,7 +1246,7 @@ int add_rr(phloat x, phloat y, phloat *z) {
     return ERR_NONE;
 }
 
-int add_rc(phloat x, phloat yre, phloat yim, phloat *zre, phloat *zim) {
+static int add_rc(phloat x, phloat yre, phloat yim, phloat *zre, phloat *zim) {
     phloat rre;
     int inf;
     rre = yre + x;
@@ -1459,7 +1262,7 @@ int add_rc(phloat x, phloat yre, phloat yim, phloat *zre, phloat *zim) {
     return ERR_NONE;
 }
 
-int add_cr(phloat xre, phloat xim, phloat y, phloat *zre, phloat *zim) {
+static int add_cr(phloat xre, phloat xim, phloat y, phloat *zre, phloat *zim) {
     phloat rre;
     int inf;
     rre = y + xre;
@@ -1475,7 +1278,7 @@ int add_cr(phloat xre, phloat xim, phloat y, phloat *zre, phloat *zim) {
     return ERR_NONE;
 }
 
-int add_cc(phloat xre, phloat xim, phloat yre, phloat yim,
+static int add_cc(phloat xre, phloat xim, phloat yre, phloat yim,
                                                     phloat *zre, phloat *zim) {
     phloat rre, rim;
     int inf;
@@ -1499,3 +1302,85 @@ int add_cc(phloat xre, phloat xim, phloat yre, phloat yim,
     *zim = rim;
     return ERR_NONE;
 }
+
+int generic_div(const vartype *px, const vartype *py, int (*completion)(int, vartype *)) {
+    if (px->type == TYPE_UNIT) {
+        if (py->type == TYPE_UNIT || py->type == TYPE_REAL) {
+            unit_ok:
+            vartype *dst;
+            int error = unit_div(px, py, &dst);
+            return completion(error, dst);
+        } else {
+            unit_fail:
+            return completion(ERR_INVALID_TYPE, NULL);
+        }
+    } else if (py->type == TYPE_UNIT) {
+        if (px->type == TYPE_REAL)
+            goto unit_ok;
+        else
+            goto unit_fail;
+    } else if ((px->type == TYPE_REALMATRIX || px->type == TYPE_COMPLEXMATRIX)
+            && (py->type == TYPE_REALMATRIX || py->type == TYPE_COMPLEXMATRIX)) {
+        return linalg_div(py, px, completion);
+    } else {
+        vartype *dst;
+        int error = map_binary(px, py, &dst, div_rr, div_rc, div_cr, div_cc);
+        return completion(error, dst);
+    }
+}
+
+int generic_mul(const vartype *px, const vartype *py, int (*completion)(int, vartype *)) {
+    if (px->type == TYPE_UNIT) {
+        if (py->type == TYPE_UNIT || py->type == TYPE_REAL) {
+            unit_ok:
+            vartype *dst;
+            int error = unit_mul(px, py, &dst);
+            return completion(error, dst);
+        } else {
+            unit_fail:
+            return completion(ERR_INVALID_TYPE, NULL);
+        }
+    } else if (py->type == TYPE_UNIT) {
+        if (px->type == TYPE_REAL)
+            goto unit_ok;
+        else
+            goto unit_fail;
+    } else if ((px->type == TYPE_REALMATRIX || px->type == TYPE_COMPLEXMATRIX)
+            && (py->type == TYPE_REALMATRIX || py->type == TYPE_COMPLEXMATRIX)) {
+        return linalg_mul(py, px, completion);
+    } else {
+        vartype *dst;
+        int error = map_binary(px, py, &dst, mul_rr, mul_rc, mul_cr, mul_cc);
+        return completion(error, dst);
+    }
+}
+
+int generic_sub(const vartype *px, const vartype *py, vartype **dst) {
+    if (px->type == TYPE_UNIT) {
+        if (py->type == TYPE_UNIT || py->type == TYPE_REAL)
+            return unit_sub(px, py, dst);
+        else
+            return ERR_INVALID_TYPE;
+    } else if (py->type == TYPE_UNIT) {
+        if (px->type == TYPE_REAL)
+            return unit_sub(px, py, dst);
+        else
+            return ERR_INVALID_TYPE;
+    } else
+        return map_binary(px, py, dst, sub_rr, sub_rc, sub_cr, sub_cc);
+}
+
+int generic_add(const vartype *px, const vartype *py, vartype **dst) {
+    if (px->type == TYPE_UNIT) {
+        if (py->type == TYPE_UNIT || py->type == TYPE_REAL)
+            return unit_add(px, py, dst);
+        else
+            return ERR_INVALID_TYPE;
+    } else if (py->type == TYPE_UNIT) {
+        if (px->type == TYPE_REAL)
+            return unit_add(px, py, dst);
+        else
+            return ERR_INVALID_TYPE;
+    } else
+        return map_binary(px, py, dst, add_rr, add_rc, add_cr, add_cc);
+}
